DROP DATABASE hotel; /*So para test*/

CREATE DATABASE hotel;

GO

USE hotel;

CREATE TABLE direccion (
	idDireccion int NOT NULL,
	pais nvarchar(50),
	provincia nvarchar(50),
	localidade nvarchar(50),
	CONSTRAINT direccion_cp PRIMARY KEY (idDireccion)
);

CREATE TABLE banco (
	idBanco int NOT NULL,
	nome nvarchar(100),
	logo nvarchar(300),
	CONSTRAINT banco_cp PRIMARY KEY (idBanco)
);

CREATE TABLE tarxeta (
	idTarxeta int NOT NULL,
	tipo nvarchar(100),
	logo nvarchar(300),
	CONSTRAINT tarxeta_cp PRIMARY KEY (idTarxeta)
);

CREATE TABLE cliente (
	idCliente int NOT NULL,	
	direccion int NOT NULL,
	domicilio nvarchar(100),
	vivenda nvarchar(100),
	tlf1 nvarchar(15),
	tlf2 nvarchar(15),
	fax nvarchar(15),
	email nvarchar(50),
	web nvarchar(200),
	banco int,
	tarxeta int,
	notas ntext,
	CONSTRAINT cliente_cp PRIMARY KEY (idCliente),
	CONSTRAINT cliente_direccion_cf FOREIGN KEY (direccion) 
		REFERENCES direccion (idDireccion) ON DELETE NO ACTION,
	CONSTRAINT cliente_banco_cf FOREIGN KEY (banco) 
		REFERENCES banco (idBanco) ON DELETE NO ACTION,
	CONSTRAINT cliente_tarxeta_cf FOREIGN KEY (tarxeta) 
		REFERENCES tarxeta (idTarxeta) ON DELETE NO ACTION
);

CREATE TABLE axencia (
	idCliente int NOT NULL,
	cif nvarchar(9) NOT NULL,
	nome nvarchar(100) NOT NULL,
	contacto nvarchar(100),
	modoPago nvarchar(7),
	numConta nvarchar(100),
	numTarxeta nvarchar(16),
	CONSTRAINT axencia_cp PRIMARY KEY (idCliente),
	CONSTRAINT axencia_modoPago_ck CHECK (modoPago IN ('Contado','Tarxeta','Bono'))	
);

CREATE TABLE empresa (
	idCliente int NOT NULL,
	cif nvarchar(9) NOT NULL,
	nome nvarchar(100) NOT NULL,
	representante nvarchar(100),
	numConta nvarchar(20),
	CONSTRAINT empresa_cp PRIMARY KEY (idCliente)
);

CREATE TABLE particular (
	idCliente int NOT NULL,
	nome nvarchar(100) NOT NULL,
	primeiroApelido nvarchar(100) NOT NULL,
	segundoApelido nvarchar(100) NOT NULL,
	dataNacemento datetime,
	tipoDocumento nvarchar(10) NOT NULL,
	numDocumento nvarchar(9) NOT NULL,
	dataExpedicion datetime,
	dataExpiracion datetime,
	sexo nvarchar(1),
	nacionalidade nvarchar(50),
	profesion nvarchar(200),
	empresa int,
	modoPago nvarchar(7),
	numConta nvarchar(100),
	numTarxeta nvarchar(16),
	codBono nvarchar(100),
	tipoCliente nvarchar(10),
	CONSTRAINT particular_cp PRIMARY KEY (idCliente),
	CONSTRAINT particular_empresa_cf FOREIGN KEY (empresa) 
		REFERENCES empresa (idCliente) ON DELETE NO ACTION,
	CONSTRAINT particular_datas_ck CHECK (dataExpedicion <= dataExpiracion),
	CONSTRAINT particular_modoPago_ck CHECK (modoPago IN ('Contado','Tarxeta','Bono')),
	CONSTRAINT particular_tipodocumento_ck CHECK (tipoDocumento IN ('DNI','Pasaporte')),
	CONSTRAINT particular_sexo_ck CHECK (sexo IN ('H','M')),
	CONSTRAINT particular_tipoCliente_ck CHECK(tipoCliente IN ('Normal', 'VIP'))
);

CREATE TABLE reserva (
	idReserva int NOT NULL,
	nomeReservador nvarchar(100),
	cifReservador nvarchar(100),
	contaReservador nvarchar(20),
	tarxetaReservador nvarchar(16),
	tlf nvarchar(15),
	estadoReserva nvarchar(13) NOT NULL,
	dataEntrada datetime NOT NULL,
	dataSaidaPrevista datetime NOT NULL,
	dataSaidaReal datetime,
	pagador int NOT NULL,
	axencia int,
	empresa int,	
	notas ntext,
	porHoras bit,
	CONSTRAINT reserva_cp PRIMARY KEY (idReserva),
	CONSTRAINT reserva_pagador_cf FOREIGN KEY (pagador) 
		REFERENCES cliente (idCliente) ON DELETE NO ACTION,
	CONSTRAINT reserva_axencia_cf FOREIGN KEY (axencia) 
		REFERENCES axencia (idCliente) ON DELETE NO ACTION,
	CONSTRAINT reserva_empresa_cf FOREIGN KEY (empresa) 
		REFERENCES empresa (idCliente) ON DELETE NO ACTION,
	CONSTRAINT reserva_datas_ck
		CHECK ((dataEntrada < dataSaidaPrevista) AND (dataEntrada < dataSaidaReal)),
	CONSTRAINT reserva_estadoreserva_ck 
		CHECK (estadoReserva IN ('Pendente','Rexistrada','Anulada','Finalizada'))
);

CREATE TABLE tipohabit (
	codTipoHabit nvarchar(5) NOT NULL,
	descricion ntext,
	CONSTRAINT tipohabit_cp PRIMARY KEY (codTipoHabit)
);

CREATE TABLE habitacion (
	codHabitacion nvarchar(10) NOT NULL,
	numero nvarchar(3) NOT NULL,
	planta tinyint,
	tipo nvarchar(5) NOT NULL,
	estadoOcupacion nvarchar(25) NOT NULL,
	estadoLimpeza nvarchar(5) NOT NULL,
	dataRevision datetime,
	fumadores bit,
	paxMaximo tinyint,
	notas ntext,
	CONSTRAINT habitacion_cp PRIMARY KEY (codHabitacion),
	CONSTRAINT habitacion_tipo_cf FOREIGN KEY (tipo) 
		REFERENCES tipohabit (codTipoHabit) ON DELETE NO ACTION,
	CONSTRAINT habitacion_estadoocupacion_ck
		CHECK (estadoOcupacion IN ('Libre','Ocupada','Bloqueada')),
	CONSTRAINT habticacion_estadolimpeza_ck
		CHECK (estadoLimpeza IN ('Limpa','Suxa'))
);

CREATE TABLE rexime (
	codRexime nvarchar(2) NOT NULL,
	descricion ntext,
	CONSTRAINT rexime_cp PRIMARY KEY (codRexime)
);

CREATE TABLE tipotarifa (
	codTipoTarifa nvarchar(3) NOT NULL,
	descricion ntext,
	CONSTRAINT tipotarifa_cp PRIMARY KEY (codTipoTarifa)
);

CREATE TABLE temporada (
	idTemporada int NOT NULL,
	nome nvarchar(10) NOT NULL,
	descricion ntext,
	CONSTRAINT temporada_cp PRIMARY KEY (idTemporada)
);

INSERT INTO temporada VALUES (1,'Ninguna','Sin temporada');

CREATE TABLE dia (
	dataDia datetime NOT NULL,
	temporada int DEFAULT 1,
	CONSTRAINT dia_cp PRIMARY KEY (dataDia),
	CONSTRAINT dia_temporada_cf FOREIGN KEY (temporada) 
		REFERENCES temporada (idTemporada) ON DELETE SET DEFAULT	
);


CREATE TABLE tarifa (
	idTarifa tinyint NOT NULL,
	prezo money,
	rexime nvarchar(2) NOT NULL,
	tipoTarifa nvarchar(3) NOT NULL,
	temporada int NOT NULL,
	CONSTRAINT tarifa_cp PRIMARY KEY (rexime, tipoTarifa, temporada, idTarifa),
	CONSTRAINT tarifa_rexime_cf FOREIGN KEY (rexime) 
		REFERENCES rexime (codRexime) ON DELETE NO ACTION,
	CONSTRAINT tarifa_tipotarifa_cf FOREIGN KEY (tipoTarifa) 
		REFERENCES tipotarifa (codTipoTarifa) ON DELETE NO ACTION,
	CONSTRAINT tarifa_temporada_cf FOREIGN KEY (temporada) 
		REFERENCES temporada (idTemporada) ON DELETE NO ACTION
);

CREATE TABLE factura (
	numFactura int NOT NULL,
	serieFactura nvarchar(5) NOT NULL DEFAULT '0',
	dataFactura datetime NOT NULL,
	nomeExpedidor nvarchar(100),
	cifExpedidor nvarchar(9),
	domicilioExpedidor nvarchar(200),
	pagador int NOT NULL,
	modoPago nvarchar(7),
	baseImponible money,
	cuotaIva money,
	total money,
	estado nvarchar(10) NOT NULL,
	observacions ntext,
	CONSTRAINT factura_cp PRIMARY KEY (numFactura, serieFactura),
	CONSTRAINT factura_pagador_cf FOREIGN KEY (pagador) 
		REFERENCES cliente (idCliente) ON DELETE NO ACTION,
	CONSTRAINT factura_modoPago_ck CHECK (modoPago IN ('Contado','Tarxeta','Bono')),
	CONSTRAINT factura_estado_dk CHECK (estado IN ('Pendente','Emitida'))
);

CREATE TABLE concepto (
	codConcepto nvarchar(5) NOT NULL,
	denominacion ntext,
	iva money,
	CONSTRAINT concepto_cp PRIMARY KEY (codConcepto)
);

INSERT INTO concepto VALUES ('Est','Estancia',7);

CREATE TABLE cargo (
	numCargo smallint NOT NULL,
	dataCargo datetime,
	prezo money,
	iva money,
	importe money,
	desconto money,
	estado nvarchar(10) NOT NULL,
	particular int,
	habitacion nvarchar(10) NOT NULL,
	numFactura int,
	serieFactura nvarchar(5),
	concepto nvarchar(5) NOT NULL,
	CONSTRAINT cargo_cp PRIMARY KEY (numCargo),
	CONSTRAINT cargo_particular_cf FOREIGN KEY (particular) 
		REFERENCES particular (idCliente) ON DELETE NO ACTION,
	CONSTRAINT cargo_habitacion_cf FOREIGN KEY (habitacion) 
		REFERENCES habitacion (codHabitacion) ON DELETE NO ACTION,
	CONSTRAINT cargo_factura_cf FOREIGN KEY (numFactura, serieFactura) 
		REFERENCES factura (numFactura, serieFactura) ON DELETE SET NULL,
	CONSTRAINT cargo_concepto_cf FOREIGN KEY (concepto) 
		REFERENCES concepto (codConcepto) ON DELETE NO ACTION,
	CONSTRAINT cargo_estado_ck CHECK (estado IN ('Pendente','Facturado'))
);

CREATE TABLE empregado (
	idEmpregado smallint NOT NULL,
	nif nvarchar(9) NOT NULL,
	nome nvarchar(100) NOT NULL,
	primeiroApelido nvarchar(100) NOT NULL,
	segundoApelido nvarchar(100) NOT NULL,
	dataNacemento datetime,
	sexo nvarchar(1),
	estadoCivil nvarchar(20),
	direccion int,
	domicilio nvarchar(100),
	vivenda nvarchar(100),
	tlf1 nvarchar(15),
	tlf2 nvarchar(15),
	posto nvarchar(200),
	dataAlta datetime,
	salario money,
	observacions ntext,
	CONSTRAINT empregado_cp PRIMARY KEY (idEmpregado),
	CONSTRAINT empregado_direccion_cf FOREIGN KEY (direccion) 
		REFERENCES direccion (idDireccion) ON DELETE NO ACTION,
	CONSTRAINT empregado_sexo_ck CHECK (sexo IN ('H','M'))
);

CREATE TABLE limpeza (
	idLimpeza int NOT NULL,
	dataLimpeza datetime NOT NULL,
	entrada bit,
	saida bit,
	toallas bit,
	sabas bit,
	desinfeccion bit,
	outra bit,
	habitacion nvarchar(10) NOT NULL,
	empregado smallint NOT NULL,
	observacions ntext,
	CONSTRAINT limpeza_cp PRIMARY KEY (habitacion, empregado, idLimpeza),
	CONSTRAINT limpeza_habitacion_cf FOREIGN KEY (habitacion)
		REFERENCES habitacion (codHabitacion) ON DELETE CASCADE,
	CONSTRAINT limpeza_empregado_cf FOREIGN KEY (empregado)
		REFERENCES empregado(idEmpregado) ON DELETE CASCADE
);
	
CREATE TABLE usuario (
	login nvarchar(20) NOT NULL,
	password nvarchar(100) NOT NULL,
	empregado smallint,
	CONSTRAINT usuario_cp PRIMARY KEY (login),
	CONSTRAINT usuario_empregado_cf FOREIGN KEY (empregado) 
		REFERENCES empregado (idEmpregado) ON DELETE SET NULL
);

INSERT INTO usuario VALUES ('admin','ceb4f32325eda6142bd65215f4c0f371',NULL);

CREATE TABLE rol (
	privilexio nvarchar(20),
	CONSTRAINT rol_cp PRIMARY KEY (privilexio)
);

INSERT INTO rol VALUES ('ROLE_ADMIN');
INSERT INTO rol VALUES ('ROLE_MANAGER');
INSERT INTO rol VALUES ('ROLE_USER');

CREATE TABLE rolusuario (
	rol nvarchar(20) NOT NULL,
	usuario nvarchar(20) NOT NULL,
	CONSTRAINT rolusuario_cp PRIMARY KEY (rol, usuario),
	CONSTRAINT rolusuario_usuario_cf FOREIGN KEY (usuario) REFERENCES usuario(login) ON DELETE CASCADE,
	CONSTRAINT rolusuario_rol_cf FOREIGN KEY (rol) REFERENCES rol (privilexio) ON DELETE CASCADE
);

INSERT INTO rolusuario VALUES ('ROLE_ADMIN','admin');
INSERT INTO rolusuario VALUES ('ROLE_MANAGER','admin');
		
CREATE TABLE caixa (
	numCaixa smallint NOT NULL,
	dataPeche datetime NOT NULL,
	numCobros smallint,
	totalTarxetas money,
	totalContado money,
	totalBonos money,
	totalAnticipos money,
	totalCaixa money,
	usuario nvarchar(20) NOT NULL,
	CONSTRAINT caixa_cp PRIMARY KEY (numCaixa),
	CONSTRAINT caixa_usuario_cf FOREIGN KEY (usuario) 
		REFERENCES usuario (login) ON DELETE NO ACTION
);

CREATE TABLE cobro (
	referencia int NOT NULL,	
	dataCobro datetime NOT NULL,
	importe money,
	modoPago nvarchar(7),
	estado nvarchar(10) NOT NULL, 
	notas ntext,
	numFactura int NOT NULL,
	serieFactura nvarchar(5) NOT NULL,
	usuario nvarchar(20) NOT NULL,
	caixa smallint,
	CONSTRAINT cobro_cp PRIMARY KEY (referencia),
	CONSTRAINT cobro_factura_cf FOREIGN KEY (numFactura, serieFactura) 
		REFERENCES factura (numFactura, serieFactura) ON DELETE NO ACTION,
	CONSTRAINT cobro_usuario_cf FOREIGN KEY (usuario) 
		REFERENCES usuario (login) ON DELETE NO ACTION,
	CONSTRAINT cobro_caixa_cf FOREIGN KEY (caixa) 
		REFERENCES caixa (numCaixa) ON DELETE SET NULL,
	CONSTRAINT cobro_modoPago_ck CHECK (modoPago IN ('Contado','Tarxeta','Bono')),
	CONSTRAINT cobro_estado_ck CHECK (estado IN ('Pechado', 'Aberto'))
);

CREATE TABLE solicitude (
	particular int NOT NULL,
	reserva int NOT NULL,
	habitacion nvarchar(10) NOT NULL,
	rexime nvarchar(2) NOT NULL,
	tipoTarifa nvarchar(3) NOT NULL,
	CONSTRAINT solicitude_cp PRIMARY KEY (particular, reserva),
	CONSTRAINT solicitude_particular_cf FOREIGN KEY (particular)
		REFERENCES particular (idCliente) ON DELETE NO ACTION,
	CONSTRAINT solicitude_reserva_cf FOREIGN KEY (reserva)
		REFERENCES reserva (idReserva) ON DELETE CASCADE,
	CONSTRAINT solicitude_habitacion_cf FOREIGN KEY (habitacion)
		REFERENCES habitacion (codHabitacion) ON DELETE NO ACTION,	
	CONSTRAINT solicitude_rexime_cf FOREIGN KEY (rexime)
		REFERENCES rexime (codRexime) ON DELETE NO ACTION,	
	CONSTRAINT solicitude_tipotarifa_cf FOREIGN KEY (tipoTarifa)
		REFERENCES tipotarifa (codTipoTarifa) ON DELETE NO ACTION
);

GO

/* Taboa de configuracin (equivalente a ficheiro plano) */

CREATE TABLE config (
	tasaExtra bit,
	cantTasa money,
	cargoTasa nvarchar(5),
	aloxPorHoras bit,
	diasLimp smallint,
	diasDesinf smallint,
	inicioDia datetime,
	moeda nvarchar(10),
	estancia nvarchar(5),
	idioma nvarchar(25),
	CONSTRAINT config_cargotasa_cf FOREIGN KEY (cargoTasa)
		REFERENCES concepto (codConcepto) ON DELETE NO ACTION,
	CONSTRAINT config_estancia_cf FOREIGN KEY (estancia) 
		REFERENCES concepto (codConcepto) ON DELETE NO ACTION	
);

INSERT INTO config (estancia, idioma) VALUES ('Est', 'es-ES');

GO