package gz.cyberbrain.cyberhotel.controller;

import gz.cyberbrain.cyberhotel.model.board.to.BoardTo;
import gz.cyberbrain.cyberhotel.model.client.to.AgencyTo;
import gz.cyberbrain.cyberhotel.model.client.to.ClientTo;
import gz.cyberbrain.cyberhotel.model.client.to.CompanyTo;
import gz.cyberbrain.cyberhotel.model.client.to.ParticularTo;
import gz.cyberbrain.cyberhotel.model.configurationfacade.delegate.ConfigurationFacadeDelegate;
import gz.cyberbrain.cyberhotel.model.pricetype.to.PriceTypeTo;
import gz.cyberbrain.cyberhotel.model.request.to.RequestTo;
import gz.cyberbrain.cyberhotel.model.reservation.to.ReservationTo;
import gz.cyberbrain.cyberhotel.model.reservationfacade.delegate.ReservationFacadeDelegate;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.CancelledReservationException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.DifferentDayException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.FinishedReservationException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.PriceNotFoundException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.RegisteredReservationException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.RoomBusyException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.RoomNotBusyException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableConceptException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableInvoiceException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableParticularException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableRoomException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnregisteredReservationException;
import gz.cyberbrain.cyberhotel.model.room.to.RoomTo;
import gz.cyberbrain.cyberhotel.model.spring.facade.SpringFacadeFactory;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InvalidIntervalException;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.TextItem;


import java.util.Calendar;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

public class CheckInController {

	private static ConfigurationFacadeDelegate config = SpringFacadeFactory.getConfigurationFacadeDelegate();
	private static ReservationFacadeDelegate delegate = SpringFacadeFactory.getReservationFacadeDelegate();
	private static HashMap<String, Object> attributes = new HashMap<String, Object>();

	private CheckInController() {}

	public static Vector checkInList() throws InternalErrorException {				
		Vector<TextItem> items = new Vector<TextItem>();

		Collection checkIns = delegate.findAllCheckIns();

		Iterator iter = checkIns.iterator();

		ReservationTo checkIn;

		for (int i=0; iter.hasNext(); i++) {

			checkIn = (ReservationTo) iter.next();
			items.add(new TextItem(checkIn.getReservationId(), checkIn.getReservatorName()));

		}

		return items;
	}

	public static void doCheckIn(Long reservationId, String reservatorName, String reservatorCif, String reservatorAccount, 
			String reservatorCard, String reservatorPhone, String reservationStatus, Calendar incomingDate, 
			Calendar expectedOutgoingDate, Calendar trueOutgoingDate, Long payer, Long agency, Long company, 
			String notes, boolean byHours) 
	throws DifferentDayException, FinishedReservationException, PriceNotFoundException, CancelledReservationException, 
	RegisteredReservationException, UnaceptableRoomException, UnaceptableConceptException, RoomBusyException, 
	UnaceptableParticularException, UnaceptableInvoiceException, RoomNotBusyException, InvalidIntervalException, 
	InstanceNotFoundException, InternalErrorException {

		delegate.checkIn(new ReservationTo(reservationId, reservatorName, reservatorCif, reservatorAccount, 
				reservatorCard, reservatorPhone, reservationStatus, incomingDate, expectedOutgoingDate, 
				trueOutgoingDate, payer, agency, company, notes, byHours));

	}

	public static void deleteCheckIn(Long reservationId) 
	throws FinishedReservationException, CancelledReservationException, UnregisteredReservationException, 
	InstanceNotFoundException, InternalErrorException {

		ReservationTo checkIn = delegate.findACheckIn(reservationId);
		delegate.removeACheckIn(checkIn);

	}

	public static Collection reservationRequests(Long reservationId) throws InternalErrorException {
		return delegate.reservationRequests(reservationId);
	}

	public static HashMap findCheckIn(Long reservationId) throws InstanceNotFoundException, InternalErrorException {

		ReservationTo checkIn = delegate.findAReservation(reservationId);

		attributes.put("id", reservationId);
		attributes.put("reservatorName", checkIn.getReservatorName());
		attributes.put("reservatorCif", checkIn.getReservatorCif());
		attributes.put("reservatorPhone", checkIn.getReservatorPhone());		
		attributes.put("reservatorAccount", checkIn.getReservatorAccount());
		attributes.put("reservatorCard", checkIn.getReservatorCard());
		attributes.put("reservationStatus", checkIn.getReservationStatus());
		attributes.put("incomingDate", checkIn.getIncomingDate());
		attributes.put("expectedOutgoingDate", checkIn.getExpectedOutgoingDate());
		attributes.put("trueOutgoingDate", checkIn.getTrueOutgoingDate());

		ClientTo payer = delegate.findAPayer(checkIn.getPayer());
		attributes.put("payer", new TextItem(payer.getClientId(), payer.getReferenceName()));

		if (checkIn.getAgency() != null) {
			AgencyTo agency = delegate.findAnAgency(checkIn.getAgency());
			attributes.put("agency", new TextItem(agency.getClientId(), agency.getName()));
		} else attributes.put("agency", new TextItem(new Long(0), ""));


		if (checkIn.getCompany() != null) {
			CompanyTo company = delegate.findACompany(checkIn.getCompany());
			attributes.put("company", new TextItem(company.getClientId(), company.getName()));
		} else attributes.put("company", new TextItem(new Long(0), ""));

		//Only for individual reservations:

		Vector request = findRequests(reservationId);

		if (request.size() == 1) {

			attributes.put("particular", ((HashMap) request.elementAt(0)).get("particular"));
			attributes.put("room", ((HashMap) request.elementAt(0)).get("room"));
			attributes.put("board", ((HashMap) request.elementAt(0)).get("board"));
			attributes.put("priceType", ((HashMap) request.elementAt(0)).get("priceType"));

		}

		attributes.put("notes", checkIn.getNotes());

		return attributes;

	}

	public static Vector findRequests(Long reservationId) {

		Collection requests;
		Vector <HashMap> requestsVector = new Vector <HashMap> ();

		try {
			requests = delegate.reservationRequests(reservationId);

			if (requests.size() == 0) throw new InternalErrorException(new Exception("Incorrect number of requests."));

			Iterator iter = requests.iterator();

			for (int i=0; iter.hasNext(); i++) {

				RequestTo request = (RequestTo) iter.next();
				HashMap <String, Object> currentRequest = new HashMap <String, Object> ();

				ParticularTo particular = delegate.findAParticular(request.getParticular());
				currentRequest.put("particular", new TextItem(particular.getClientId(), particular.getReferenceName()));

				RoomTo room = config.findRoom(request.getRoom());		
				currentRequest.put("room", new TextItem(room.getRoomCode(), room.getNumber()));

				BoardTo board = config.findBoard(request.getBoard());
				currentRequest.put("board", new TextItem(board.getBoardCode(), board.getDescription()));

				PriceTypeTo priceType = config.findPriceType(request.getPriceType());
				currentRequest.put("priceType", new TextItem(priceType.getPriceTypeCode(), priceType.getDescription()));

				requestsVector.add(currentRequest);

			}

		} catch (InstanceNotFoundException e) {
			e.printStackTrace();
		} catch (InternalErrorException e) {			
			e.printStackTrace();
		} 

		return requestsVector;

	}

}
