package gz.cyberbrain.cyberhotel.controller;

import gz.cyberbrain.cyberhotel.controller.util.ErrorItem;
import gz.cyberbrain.cyberhotel.controller.util.ErrorsHandler;
import gz.cyberbrain.cyberhotel.model.client.to.ParticularTo;
import gz.cyberbrain.cyberhotel.model.concept.to.ConceptTo;
import gz.cyberbrain.cyberhotel.model.configurationfacade.delegate.ConfigurationFacadeDelegate;
import gz.cyberbrain.cyberhotel.model.owe.to.OweTo;
import gz.cyberbrain.cyberhotel.model.reservationfacade.delegate.ReservationFacadeDelegate;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.RoomNotBusyException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableConceptException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableInvoiceException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableParticularException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableRoomException;
import gz.cyberbrain.cyberhotel.model.room.to.RoomTo;
import gz.cyberbrain.cyberhotel.model.spring.facade.SpringFacadeFactory;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.TableItem;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.TextItem;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

public class OweController {

	private static ConfigurationFacadeDelegate config = SpringFacadeFactory.getConfigurationFacadeDelegate();
	private static ReservationFacadeDelegate delegate = SpringFacadeFactory.getReservationFacadeDelegate();
	private static HashMap<String, Object> attributes = new HashMap<String, Object>();

	private OweController() {}

	public static Vector owesByRoomAndParticular(String roomCode, Long particularId) {				

		Vector<TableItem> items = new Vector<TableItem> ();

		try {


			Collection owes;

			owes = delegate.findOwesByRoomAndParticular(roomCode, particularId);

			Iterator iter = owes.iterator();

			OweTo owe;
			RoomTo room;
			ParticularTo particular;
			ConceptTo concept;

			Object[] columns = new Object[8];
			Serializable[] id = new Serializable[1];

			for (int i=0; iter.hasNext(); i++) {

				columns = new Object[8];
				id = new Serializable[1];

				owe = (OweTo) iter.next();

				room = config.findRoom(owe.getRoom());
				particular = delegate.findAParticular(owe.getParticular());
				concept = config.findConcept(owe.getConcept());				

				columns[0] = owe.getOweDate();
				columns[1] = new TextItem(room.getRoomCode(), room.getNumber());
				columns[2] = new TextItem(particular.getClientId(), particular.getReferenceName());
				columns[3] = new TextItem(concept.getConceptCode(), concept.getDenomination());
				columns[4] = owe.getPrice();
				columns[5] = owe.getVat();
				columns[6] = owe.getAmount();
				columns[7] = owe.getStatus();

				id[0] = owe.getOweNumber();			

				items.add(new TableItem(id, columns));

			}		

		} catch (UnaceptableRoomException e) {
			e.printStackTrace();
		} catch (UnaceptableParticularException e) {
			e.printStackTrace();
		} catch (InstanceNotFoundException e) {
			e.printStackTrace();
		} catch (InternalErrorException e) {
			e.printStackTrace();
		}

		return items;

	}
	
	public static Vector owesByRoom(String roomCode) {				

		Vector<TableItem> items = new Vector<TableItem> ();

		try {


			Collection owes;

			owes = delegate.findOwesByRoom(roomCode);

			Iterator iter = owes.iterator();

			OweTo owe;
			RoomTo room;
			ParticularTo particular;
			ConceptTo concept;

			Object[] columns = new Object[8];
			Serializable[] id = new Serializable[1];

			for (int i=0; iter.hasNext(); i++) {

				columns = new Object[8];
				id = new Serializable[2];

				owe = (OweTo) iter.next();

				room = config.findRoom(owe.getRoom());
				particular = delegate.findAParticular(owe.getParticular());
				concept = config.findConcept(owe.getConcept());

				columns[0] = owe.getOweDate();
				columns[1] = new TextItem(room.getRoomCode(), room.getNumber());
				columns[2] = new TextItem(particular.getClientId(), particular.getReferenceName());
				columns[3] = new TextItem(concept.getConceptCode(), concept.getDenomination());
				columns[4] = owe.getPrice();
				columns[5] = owe.getVat();
				columns[6] = owe.getAmount();
				columns[7] = owe.getStatus();

				id[0] = owe.getOweNumber();			

				items.add(new TableItem(id, columns));

			}		

		} catch (UnaceptableRoomException e) {
			e.printStackTrace();
		} catch (InstanceNotFoundException e) {
			e.printStackTrace();
		} catch (InternalErrorException e) {
			e.printStackTrace();
		}

		return items;

	}
	
	public static Vector owesByParticular(Long particularId) {				

		Vector<TableItem> items = new Vector<TableItem> ();

		try {


			Collection owes;

			owes = delegate.findOwesByParticular(particularId);

			Iterator iter = owes.iterator();

			OweTo owe;
			RoomTo room;
			ParticularTo particular;
			ConceptTo concept;

			Object[] columns = new Object[8];
			Serializable[] id = new Serializable[2];

			for (int i=0; iter.hasNext(); i++) {

				columns = new Object[8];
				id = new Serializable[1];

				owe = (OweTo) iter.next();

				room = config.findRoom(owe.getRoom());
				particular = delegate.findAParticular(owe.getParticular());
				concept = config.findConcept(owe.getConcept());

				columns[0] = owe.getOweDate();
				columns[1] = new TextItem(room.getRoomCode(), room.getNumber());
				columns[2] = new TextItem(particular.getClientId(), particular.getReferenceName());
				columns[3] = new TextItem(concept.getConceptCode(), concept.getDenomination());
				columns[4] = owe.getPrice();
				columns[5] = owe.getVat();
				columns[6] = owe.getAmount();
				columns[7] = owe.getStatus();

				id[0] = owe.getOweNumber();			

				items.add(new TableItem(id, columns));

			}		

		} catch (UnaceptableParticularException e) {
			e.printStackTrace();
		} catch (InstanceNotFoundException e) {
			e.printStackTrace();
		} catch (InternalErrorException e) {
			e.printStackTrace();
		}

		return items;

	}

	public static void createOwe(Long oweNumber, Calendar oweDate, BigDecimal price, BigDecimal vat, BigDecimal amount,
			BigDecimal discount, String status, Long particular, String room, Long invoiceNumber, String invoiceSerial,
			String concept) {
		
		try {
			delegate.oweAddition(new OweTo(oweNumber, oweDate, price, vat, amount, discount, status, particular, room, invoiceNumber, invoiceSerial, concept));
		} catch (UnaceptableRoomException e) {
			ErrorsHandler.clear();
			ErrorsHandler.add(new ErrorItem("roomUnaceptable"));
		} catch (UnaceptableConceptException e) {
			ErrorsHandler.clear();
			ErrorsHandler.add(new ErrorItem("invoice"));
		} catch (UnaceptableParticularException e) {
			ErrorsHandler.clear();
			ErrorsHandler.add(new ErrorItem("concept"));
		} catch (UnaceptableInvoiceException e) {
			ErrorsHandler.clear();
			ErrorsHandler.add(new ErrorItem("particular"));
		} catch (RoomNotBusyException e) {
			ErrorsHandler.clear();
			ErrorsHandler.add(new ErrorItem("roomNotBusy"));
		} catch (InstanceNotFoundException e) {
			e.printStackTrace();
		} catch (InternalErrorException e) {
			e.printStackTrace();
		} 
	}

	public static void updateOwe(Long oweNumber, Calendar oweDate, BigDecimal price, BigDecimal vat, BigDecimal amount,
			BigDecimal discount, String status, Long particular, String room, Long invoiceNumber, String invoiceSerial,
			String concept)  {		
		try {
			delegate.oweUpdate(new OweTo(oweNumber, oweDate, price, vat, amount, discount, status, particular, room, invoiceNumber, invoiceSerial, concept));
		} catch (InternalErrorException e) {
			e.printStackTrace();
		}
	}

	public static void deleteOwe(Long oweNumber) {
		OweTo owe;
		try {
			owe = delegate.findAnOwe(oweNumber);			
			delegate.removeAnOwe(owe);
		} catch (InstanceNotFoundException e) {
			e.printStackTrace();
		} catch (InternalErrorException e) {
			e.printStackTrace();
		}

	}

	public static HashMap findOwe(Long oweNumber) throws InstanceNotFoundException, InternalErrorException {


		OweTo owe = delegate.findAnOwe(oweNumber);

		attributes.put("oweNumber", owe.getOweNumber());
		attributes.put("oweDate", owe.getOweDate());
		attributes.put("price", owe.getPrice());
		attributes.put("vat", owe.getVat());
		attributes.put("amount", owe.getAmount());
		attributes.put("discount", owe.getDiscount());
		attributes.put("status", owe.getStatus());
		attributes.put("particular", owe.getParticular());
		attributes.put("room", owe.getRoom());
		attributes.put("invoiceNumber", owe.getInvoiceNumber());
		attributes.put("invoiceSerial", owe.getInvoiceSerial());
		attributes.put("concept", owe.getConcept());

		return attributes;

	}

}
