package gz.cyberbrain.cyberhotel.model.reservationfacade.spring;

import java.util.Calendar;
import java.util.Collection;

import gz.cyberbrain.cyberhotel.model.address.dao.AddressDao;
import gz.cyberbrain.cyberhotel.model.bank.dao.BankDao;
import gz.cyberbrain.cyberhotel.model.board.dao.BoardDao;
import gz.cyberbrain.cyberhotel.model.card.dao.CardDao;
import gz.cyberbrain.cyberhotel.model.client.dao.ClientDao;
import gz.cyberbrain.cyberhotel.model.client.to.AgencyTo;
import gz.cyberbrain.cyberhotel.model.client.to.ClientTo;
import gz.cyberbrain.cyberhotel.model.client.to.CompanyTo;
import gz.cyberbrain.cyberhotel.model.client.to.ParticularTo;
import gz.cyberbrain.cyberhotel.model.owe.dao.OweDao;
import gz.cyberbrain.cyberhotel.model.owe.to.OweTo;
import gz.cyberbrain.cyberhotel.model.price.dao.PriceDao;
import gz.cyberbrain.cyberhotel.model.pricetype.dao.PriceTypeDao;
import gz.cyberbrain.cyberhotel.model.request.dao.RequestDao;
import gz.cyberbrain.cyberhotel.model.reservation.dao.ReservationDao;
import gz.cyberbrain.cyberhotel.model.reservation.to.ReservationTo;
import gz.cyberbrain.cyberhotel.model.reservationfacade.delegate.ReservationFacadeDelegate;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.ClientPayerException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.AgencyWithReservationsException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.CancelledReservationException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.CompanyWithReservationsException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.DifferentDayException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.FinishedReservationException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.InvalidDocumentTypeException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.InvalidPayModeException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.InvalidSexException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.ClientWithInvoicesException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.ParticularWithOwesException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.ParticularWithReservationsException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.PendingReservationException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.PriceNotFoundException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.RegisteredReservationException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.RoomBusyException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.RoomNotBusyException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableAddressException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableAgencyException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableBankException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableParticularException;
import gz.cyberbrain.cyberhotel.model.concept.dao.ConceptDao;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.UnaceptableBoardException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableCardException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableCompanyException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableConceptException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableEmployeeException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableInvoiceException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptablePayerException;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.UnaceptablePriceTypeException;
import gz.cyberbrain.cyberhotel.model.day.dao.DayDao;
import gz.cyberbrain.cyberhotel.model.employee.dao.EmployeeDao;
import gz.cyberbrain.cyberhotel.model.invoice.dao.InvoiceDao;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableRoomException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnavailableRoomException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnregisteredReservationException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions.BlockARoomAction;
import gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions.CheckInAction;
import gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions.CheckOutAction;
import gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions.FindOwesByParticularAction;
import gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions.FindOwesByRoomAction;
import gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions.RemoveACompanyAction;
import gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions.RemoveAParticularAction;
import gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions.RemoveAnAgencyAction;
import gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions.DoAWashAction;
import gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions.FindACheckInAction;
import gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions.FindACheckOutAction;
import gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions.FindOwesByRoomAndParticularAction;
import gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions.InsertACompanyAction;
import gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions.InsertAParticularAction;
import gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions.InsertAnAgencyAction;
import gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions.OweAdditionAction;
import gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions.RemoveACheckInAction;
import gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions.RemoveAReservationAction;
import gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions.MakeAReservationAction;
import gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions.UnblockARoomAction;
import gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions.UpdateACompanyAction;
import gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions.UpdateAParticularAction;
import gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions.UpdateAReservationAction;
import gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions.UpdateAnAgencyAction;
import gz.cyberbrain.cyberhotel.model.room.dao.RoomDao;
import gz.cyberbrain.cyberhotel.model.room.to.RoomTo;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InvalidIntervalException;
import gz.cyberbrain.cyberhotel.model.wash.dao.WashDao;
import gz.cyberbrain.cyberhotel.model.wash.to.WashTo;

public class ReservationSpringFacadeDelegate implements ReservationFacadeDelegate {

	private static final long serialVersionUID = -339604673306020774L;	

	private AddressDao addressDao;
	private BankDao bankDao;
	private CardDao cardDao;
	private BoardDao boardDao;
	private ClientDao agencyDao;
	private ClientDao companyDao;
	private ClientDao particularDao;
	private PriceTypeDao priceTypeDao;
	private RequestDao requestDao;
	private ReservationDao reservationDao;
	private RoomDao roomDao;
	private ConceptDao conceptDao;	
	private PriceDao priceDao;
	private DayDao dayDao;
	private OweDao oweDao;
	private InvoiceDao invoiceDao;
	private EmployeeDao employeeDao;
	private WashDao washDao;

	public void setAddressDao(AddressDao addressDao) {
		this.addressDao = addressDao;
	}

	public void setBankDao(BankDao bankDao) {
		this.bankDao = bankDao;
	}

	public void setCardDao(CardDao cardDao) {
		this.cardDao = cardDao;
	}

	public void setBoardDao(BoardDao boardDao) {
		this.boardDao = boardDao;		
	}

	public void setAgencyDao(ClientDao agencyDao) {
		this.agencyDao = agencyDao;
	}

	public void setCompanyDao(ClientDao companyDao) {
		this.companyDao = companyDao;
	}

	public void setParticularDao(ClientDao particularDao) {
		this.particularDao = particularDao;
	}		

	public void setPriceTypeDao(PriceTypeDao priceTypeDao) {
		this.priceTypeDao = priceTypeDao;
	}

	public void setRequestDao(RequestDao requestDao) {
		this.requestDao = requestDao;
	}

	public void setReservationDao(ReservationDao reservationDao) {
		this.reservationDao = reservationDao;
	}

	public void setRoomDao(RoomDao roomDao) {
		this.roomDao = roomDao;
	}

	public void setConceptDao(ConceptDao conceptDao) {
		this.conceptDao = conceptDao;
	}

	public void setPriceDao(PriceDao priceDao) {
		this.priceDao = priceDao;
	}
	public void setDayDao(DayDao dayDao) {
		this.dayDao = dayDao;
	}

	public void setOweDao(OweDao oweDao) {
		this.oweDao = oweDao;
	}

	public void setInvoiceDao(InvoiceDao invoiceDao) {
		this.invoiceDao = invoiceDao;
	}

	public void setEmployeeDao (EmployeeDao employeeDao) {
		this.employeeDao = employeeDao;
	}

	public void setWashDao(WashDao washDao) {
		this.washDao = washDao;
	}

	public ParticularTo insertAParticular(ParticularTo particularTo) 
	throws InvalidSexException, InvalidDocumentTypeException, InvalidPayModeException, 
	UnaceptableAddressException, UnaceptableCompanyException,
	UnaceptableBankException, UnaceptableCardException,
	InvalidIntervalException, InternalErrorException {

		InsertAParticularAction action = new InsertAParticularAction(particularTo, addressDao, bankDao, cardDao, 
				companyDao, particularDao);
		return action.execute();

	}

	public ParticularTo findAParticular(Long clientId) 
	throws InstanceNotFoundException, InternalErrorException {

		//ClientDao particularDao = SpringDaoFactory.getParticularDao();
		return (ParticularTo) particularDao.find(clientId);

	}

	public void updateAParticular(ParticularTo particularTo) 
	throws InvalidSexException, InvalidDocumentTypeException, InvalidPayModeException, 
	UnaceptableAddressException, UnaceptableCompanyException,
	UnaceptableBankException, UnaceptableCardException,
	InvalidIntervalException, InternalErrorException {

		UpdateAParticularAction action = new UpdateAParticularAction(particularTo, addressDao, bankDao, cardDao, 
				companyDao, particularDao);
		action.execute();

	}

	public void removeAParticular(ParticularTo particularTo) 
	throws ParticularWithOwesException, ParticularWithReservationsException, 
	ClientPayerException, ClientWithInvoicesException, InternalErrorException {

		RemoveAParticularAction action = new RemoveAParticularAction(particularTo, particularDao);
		action.execute();

	}

	public Collection showParticulars() throws InternalErrorException {

		//ClientDao particularDao = SpringDaoFactory.getParticularDao();
		return particularDao.findAll();

	}

	public AgencyTo insertAnAgency (AgencyTo agencyTo) 
	throws InvalidPayModeException, UnaceptableAddressException, 
	UnaceptableBankException, UnaceptableCardException,
	InternalErrorException {

		InsertAnAgencyAction action = new InsertAnAgencyAction(agencyTo, addressDao, bankDao, cardDao, agencyDao);
		return action.execute();

	}

	public AgencyTo findAnAgency(Long clientId) 
	throws InstanceNotFoundException, InternalErrorException {

		//ClientDao agencyDao = SpringDaoFactory.getAgencyDao();
		return (AgencyTo) agencyDao.find(clientId);

	}

	public void updateAnAgency (AgencyTo agencyTo) 
	throws InvalidPayModeException, UnaceptableAddressException, 
	UnaceptableBankException, UnaceptableCardException,
	InternalErrorException {

		UpdateAnAgencyAction action = new UpdateAnAgencyAction(agencyTo, addressDao, bankDao, cardDao, agencyDao);
		action.execute();
	}

	public void removeAnAgency (AgencyTo agencyTo) 
	throws AgencyWithReservationsException, ClientPayerException,
	ClientWithInvoicesException, InternalErrorException {

		RemoveAnAgencyAction action = new RemoveAnAgencyAction(agencyTo, agencyDao);
		action.execute();

	}

	public Collection showAgencies() throws InternalErrorException {

		//ClientDao clientDao = SpringDaoFactory.getAgencyDao();
		return agencyDao.findAll();

	}

	public CompanyTo insertACompany (CompanyTo companyTo) 
	throws UnaceptableAddressException, UnaceptableBankException, InternalErrorException {

		InsertACompanyAction action = new InsertACompanyAction(companyTo, addressDao, bankDao, companyDao);
		return action.execute();

	}

	public CompanyTo findACompany(Long clientId) 
	throws InstanceNotFoundException, InternalErrorException {

		//ClientDao companyDao = SpringDaoFactory.getCompanyDao();
		return (CompanyTo) companyDao.find(clientId);

	}

	public void updateACompany (CompanyTo companyTo) 
	throws UnaceptableAddressException, UnaceptableBankException, InternalErrorException {

		UpdateACompanyAction action = new UpdateACompanyAction(companyTo, addressDao, bankDao, companyDao);
		action.execute();

	}

	public void removeACompany(CompanyTo companyTo) 
	throws CompanyWithReservationsException, ClientPayerException,
	ClientWithInvoicesException, InternalErrorException {

		RemoveACompanyAction action = new RemoveACompanyAction(companyTo, companyDao);
		action.execute();

	}

	public Collection showCompanies() throws InternalErrorException {

		//ClientDao clientDao = SpringDaoFactory.getCompanyDao();
		return companyDao.findAll();
	}

	public ClientTo findAPayer(Long clientId) throws InstanceNotFoundException, InternalErrorException {

		ClientTo clientTo = null;

		try {

			//ClientDao particularDao = SpringDaoFactory.getParticularDao();
			clientTo = particularDao.find(clientId);

		} catch (InstanceNotFoundException e) {}

		try {

			//ClientDao agencyDao = SpringDaoFactory.getAgencyDao();
			clientTo = agencyDao.find(clientId);

		} catch (InstanceNotFoundException e) {}

		try {

			//ClientDao companyDao = SpringDaoFactory.getCompanyDao();
			clientTo = companyDao.find(clientId);

		} catch (InstanceNotFoundException e) {}

		if (clientTo != null) return clientTo;
		else throw new InstanceNotFoundException(clientTo, "Payer not found.");

	}

	public ReservationTo makeAReservation(ReservationTo reservationTo, 
			Collection requestsTo) 
	throws UnaceptableParticularException, UnaceptableBoardException, UnaceptablePriceTypeException, 
	UnaceptableAgencyException, UnaceptablePayerException, UnaceptableCompanyException, UnaceptableRoomException, 
	UnavailableRoomException, InvalidIntervalException, InternalErrorException {

		MakeAReservationAction action = new MakeAReservationAction(reservationTo, requestsTo, boardDao, agencyDao, companyDao, particularDao, priceTypeDao, requestDao, reservationDao, roomDao);
		return action.execute();

	}

	public ReservationTo findAReservation(Long reservationId) throws InstanceNotFoundException, InternalErrorException {

		//ReservationDao reservationDao = SpringDaoFactory.getReservationDao();
		return reservationDao.find(reservationId);

	}

	public void updateAReservation(ReservationTo reservationTo, Collection requestsTo)
	throws UnaceptableParticularException, UnaceptableBoardException, UnaceptablePriceTypeException, 
	UnaceptableAgencyException, UnaceptablePayerException, UnaceptableCompanyException, UnaceptableRoomException, 
	UnavailableRoomException, InvalidIntervalException, InstanceNotFoundException, InternalErrorException {

		UpdateAReservationAction action = new UpdateAReservationAction(reservationTo, requestsTo, boardDao, agencyDao, companyDao, particularDao, priceTypeDao, requestDao, reservationDao, roomDao);
		action.execute();

	}

	public Collection showUnregisteredReservations() throws InternalErrorException {
		return reservationDao.findByStatus("Pendente");
	}

	public Collection showRegisteredReservations() throws InternalErrorException {
		return reservationDao.findByStatus("Rexistrada");
	}

	public Collection showCancelledReservations() throws InternalErrorException {
		return reservationDao.findByStatus("Anulada");
	}

	public Collection showFinishedReservations() throws InternalErrorException {
		return reservationDao.findByStatus("Finalizada");
	}

	public Collection showReservations() throws InternalErrorException {

		//ReservationDao reservationDao = SpringDaoFactory.getReservationDao();
		return reservationDao.findAll();

	}

	public void removeAReservation(ReservationTo reservationTo) 
	throws RegisteredReservationException, InternalErrorException {

		RemoveAReservationAction action = new RemoveAReservationAction(reservationTo, reservationDao, requestDao);
		action.execute();

	}

	public Collection reservationRequests(Long reservationId) throws InternalErrorException {

		//ReservationDao reservationDao = SpringDaoFactory.getReservationDao();
		return reservationDao.findRequests(reservationId);

	}

	public void checkIn(ReservationTo reservationTo) 
	throws DifferentDayException, FinishedReservationException, PriceNotFoundException, 
	CancelledReservationException, RegisteredReservationException, 
	UnaceptableRoomException, UnaceptableConceptException, RoomBusyException,
	UnaceptableParticularException, UnaceptableInvoiceException, RoomNotBusyException,
	InvalidIntervalException, InstanceNotFoundException, InternalErrorException {

		CheckInAction action = new CheckInAction(reservationTo, reservationDao, roomDao, conceptDao, dayDao, priceDao);
		action.execute();

	}

	public ReservationTo findACheckIn (Long reservationId) 
	throws InstanceNotFoundException, InternalErrorException {

		FindACheckInAction action = new FindACheckInAction(reservationId, reservationDao);
		return action.execute();

	}

	public Collection findAllCheckIns() throws InternalErrorException {

		//ReservationDao reservationDao = SpringDaoFactory.getReservationDao();
		return reservationDao.findByStatus("Rexistrada");

	}

	public void removeACheckIn(ReservationTo reservationTo) 
	throws FinishedReservationException, CancelledReservationException, UnregisteredReservationException, 
	InstanceNotFoundException, InternalErrorException {

		RemoveACheckInAction action = new RemoveACheckInAction(reservationTo, reservationDao, oweDao, roomDao);
		action.execute();

	}

	public void checkOut (ReservationTo reservationTo, Calendar trueOutgoingDate) 
	throws  DifferentDayException, PendingReservationException, PriceNotFoundException, 
	CancelledReservationException, FinishedReservationException,    
	InvalidIntervalException, InstanceNotFoundException, InternalErrorException, 
	UnaceptableRoomException, UnaceptableConceptException, UnaceptableParticularException, 
	UnaceptableInvoiceException, RoomNotBusyException {

		CheckOutAction action = new CheckOutAction(reservationTo, trueOutgoingDate, reservationDao, roomDao, conceptDao, dayDao, priceDao, oweDao);
		action.execute();

	}

	public ReservationTo findACheckOut (Long reservationId) 
	throws InstanceNotFoundException, InternalErrorException {

		FindACheckOutAction action = new FindACheckOutAction(reservationId, reservationDao);
		return action.execute();

	}

	public Collection findAllCheckOuts() throws InternalErrorException {

		//ReservationDao reservationDao = SpringDaoFactory.getReservationDao();
		return reservationDao.findByStatus("Finalizada");

	}

	/*public Long nextOweNumber(Long reservationId) throws InternalErrorException {
		
		return oweDao.nextOwe(reservationId);
		
	}*/

	public OweTo oweAddition (OweTo oweTo) 
	throws UnaceptableRoomException, UnaceptableConceptException, UnaceptableParticularException, 
	UnaceptableInvoiceException, RoomNotBusyException, InstanceNotFoundException, InternalErrorException {

		OweAdditionAction action = new OweAdditionAction(oweTo, roomDao, invoiceDao, conceptDao, particularDao, oweDao);
		return action.execute();

	}

	public void oweUpdate(OweTo oweTo) throws InternalErrorException {
		oweDao.update(oweTo);		
	}

	public OweTo findAnOwe(Long oweNumber)
	throws InstanceNotFoundException, InternalErrorException {

		return oweDao.find(oweNumber);

	}

	public Collection findOwesByRoom(String roomCode) 
	throws UnaceptableRoomException, InternalErrorException {

		FindOwesByRoomAction action = new FindOwesByRoomAction(roomCode,  roomDao, oweDao);
		return action.execute();

	}

	public Collection findOwesByParticular(Long particularId) throws UnaceptableParticularException, InternalErrorException {

		FindOwesByParticularAction action = new FindOwesByParticularAction(particularId,  particularDao, oweDao);
		return action.execute();

	}

	public Collection findOwesByRoomAndParticular(String roomCode, Long particularId) 
	throws UnaceptableRoomException, UnaceptableParticularException, InternalErrorException {

		FindOwesByRoomAndParticularAction action = new FindOwesByRoomAndParticularAction(roomCode, particularId, roomDao, particularDao, oweDao);
		return action.execute();

	}

	public void removeAnOwe (OweTo oweTo) throws InternalErrorException {

		//OweDao oweDao = SpringDaoFactory.getOweDao();
		oweDao.remove(oweTo);

	}

	public void blockARoom (RoomTo roomTo) throws RoomBusyException, InternalErrorException {

		BlockARoomAction action = new BlockARoomAction(roomTo, roomDao);
		action.execute();

	}

	public void unblockARoom (RoomTo roomTo) throws InternalErrorException {

		UnblockARoomAction action = new UnblockARoomAction(roomTo, roomDao);
		action.execute();
	}

	public WashTo doAWash (WashTo washTo) throws UnaceptableEmployeeException, UnaceptableRoomException, InternalErrorException {

		DoAWashAction action = new DoAWashAction(washTo, roomDao, employeeDao, washDao);
		return action.execute();

	}

}
