package gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Collection;
import java.util.Iterator;

import gz.cyberbrain.cyberhotel.model.concept.dao.ConceptDao;
import gz.cyberbrain.cyberhotel.model.concept.to.ConceptTo;
import gz.cyberbrain.cyberhotel.model.day.dao.DayDao;
import gz.cyberbrain.cyberhotel.model.owe.to.OweTo;
import gz.cyberbrain.cyberhotel.model.price.dao.PriceDao;
import gz.cyberbrain.cyberhotel.model.price.to.PriceTo;
import gz.cyberbrain.cyberhotel.model.request.to.RequestTo;
import gz.cyberbrain.cyberhotel.model.reservation.dao.ReservationDao;
import gz.cyberbrain.cyberhotel.model.reservation.to.ReservationTo;
import gz.cyberbrain.cyberhotel.model.reservationfacade.delegate.ReservationFacadeDelegate;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.CancelledReservationException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.DifferentDayException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.FinishedReservationException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.PriceNotFoundException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.RegisteredReservationException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.RoomBusyException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.RoomNotBusyException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableConceptException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableInvoiceException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableParticularException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableRoomException;
import gz.cyberbrain.cyberhotel.model.room.dao.RoomDao;
import gz.cyberbrain.cyberhotel.model.room.to.RoomTo;
import gz.cyberbrain.cyberhotel.model.spring.facade.SpringFacadeFactory;
import gz.cyberbrain.cyberhotel.model.util.config.ConfigurationHandler;
import gz.cyberbrain.cyberhotel.model.util.dates.DateOperations;
import gz.cyberbrain.cyberhotel.model.util.dates.Interval;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InvalidIntervalException;
import gz.cyberbrain.cyberhotel.model.util.money.MoneyOperations;

public class CheckInAction {

	private ReservationTo reservationTo;
	private ReservationDao reservationDao;
	private RoomDao roomDao;
	private ConceptDao conceptDao;
	private DayDao dayDao;
	private PriceDao priceDao;

	public CheckInAction(ReservationTo reservationTo, ReservationDao reservationDao, RoomDao roomDao,
			ConceptDao conceptDao, DayDao dayDao, PriceDao priceDao) {		
		this.reservationTo = reservationTo;
		this.reservationDao = reservationDao;
		this.roomDao = roomDao;
		this.conceptDao = conceptDao;
		this.dayDao = dayDao;
		this.priceDao = priceDao;

	}

	public void execute() throws DifferentDayException, FinishedReservationException, 
	CancelledReservationException, RegisteredReservationException, PriceNotFoundException, 
	UnaceptableParticularException, RoomNotBusyException, RoomBusyException,
	InvalidIntervalException, InstanceNotFoundException, InternalErrorException, 
	UnaceptableRoomException, UnaceptableConceptException, UnaceptableInvoiceException {

		if (reservationTo.getReservationStatus().equals("Rexistrada")) 
			throw new RegisteredReservationException(reservationTo);

		if (reservationTo.getReservationStatus().equals("Anulada"))
			throw new CancelledReservationException(reservationTo);

		if (reservationTo.getReservationStatus().equals("Finalizada")) 
			throw new FinishedReservationException(reservationTo);				

		reservationTo.setReservationStatus("Rexistrada");				

		Collection requests = reservationDao.findRequests(reservationTo.getReservationId());

		Iterator iter = requests.iterator();		

		RoomTo updatedRoom;

		for (int i=0; iter.hasNext();i++) {
			updatedRoom = roomDao.find(((RequestTo) iter.next()).getRoom());
			if (updatedRoom.getOccupancyStatus().equals("Ocupada")) throw new RoomBusyException(updatedRoom);			
		}

		iter = requests.iterator();		

		for (int i=0; iter.hasNext();i++) {
			updatedRoom = roomDao.find(((RequestTo) iter.next()).getRoom());
			updatedRoom.setOccupancyStatus("Ocupada");
			roomDao.update(updatedRoom);
		}

		//System.out.println(">>>>>>>>>>>UPDATE FAISE 1<<<<<<<<<<");

		reservationDao.update(reservationTo);

		//System.out.println(">>>>>>>>>>>UPDATE FAISE 2<<<<<<<<<<");

		/* Owes assignment*/

		ConfigurationHandler.init();

		ConceptTo conceptTo = conceptDao.find(ConfigurationHandler.STAY_CONCEPT);
		BigDecimal vat = conceptTo.getVat();						

		String board = null;
		String priceType = null;
		Long season = null;
		Long priceId;

		BigDecimal price;
		BigDecimal amount;
		BigDecimal discount = new BigDecimal(0); //No offers implemented
		String status = "Pendente";
		Long particular;
		String room;
		Long invoiceNumber = null;
		String invoiceSerial = null;
		String concept = conceptTo.getConceptCode();
		//Long reservation = reservationTo.getReservationId();

		Interval interval = new Interval(reservationTo.getIncomingDate(), reservationTo.getExpectedOutgoingDate());
		System.out.println("CHeckinaction5");
		Collection dates;

		//If byHours owes per hours, else owes per day. 

		if (reservationTo.getByHours()) {
			//System.out.println("XERO POR HORAS");
			priceId = new Long(2);
			if ((interval.getStart()).get(Calendar.DAY_OF_YEAR) != (interval.getEnd()).get(Calendar.DAY_OF_YEAR)) 
				throw new DifferentDayException(reservationTo);
			dates = DateOperations.getIntervalHours(interval);
		} else {
			//System.out.println("XERO POR DAS");
			dates = DateOperations.getIntervalDates(interval);
			priceId = new Long(1);
		}
		System.out.println("Dates_size: " + dates.size());
		Iterator days = dates.iterator(); //Days of the reservation

		// Search for prices and create owes (price value depends on current date).

		Calendar currentDate;
		RequestTo currentRequest;
		PriceTo currentPrice;

		//System.out.println("DATES_SIZE: " + dates.size());

		//============ ELIMINAR ESTO DESDE AQU ===================

		/* Creamos dias mentres non traballamos con calendarios */

		/* Days of August */

		/*		Collection <DayTo> tmpDays = new ArrayList<DayTo>();

		Calendar date = Calendar.getInstance();

		for (int i=1; i<32; i++) {

			date.set(2007,7,i,0,0,0);
			tmpDays.add(new DayTo(date, new Long(1)));				

		}

		Iterator tmpIter = tmpDays.iterator();

		for (int i=0; tmpIter.hasNext(); i++) dayDao.create((DayTo)tmpIter.next());
		 */
		//=============== ATA AQU ===================================

		for (int i=0; days.hasNext(); i++) {

			currentDate = (Calendar) days.next();			
			System.out.println("CheckInAction: Dia " + currentDate.get(Calendar.DATE) + "/" + currentDate.get(Calendar.MONTH) + "/" +currentDate.get(Calendar.YEAR));
			try {
				season = (dayDao.find(currentDate)).getSeason();
			} catch (InstanceNotFoundException e) {
				throw new PriceNotFoundException(priceId, board, priceType, season);
			}

			iter = requests.iterator();

			//System.out.println("PERSOAS: " + requests.size());

			for (int j=0; iter.hasNext();j++) {

				currentRequest = (RequestTo) iter.next(); 

				board = (currentRequest).getBoard();

				priceType = (currentRequest).getPriceType();
				System.out.println("PriceTo(" + priceId + ", " + board + ", " + priceType + ", " + season + ")");
				try {				
					currentPrice = priceDao.find(priceId, board, priceType, season);
				} catch (InstanceNotFoundException e) {
					throw new PriceNotFoundException(priceId, board, priceType, season);
				}

				price = currentPrice.getValue();		
				amount = MoneyOperations.getPriceWithVat(price,vat);

				particular = currentRequest.getParticular();
				room = currentRequest.getRoom();

				ReservationFacadeDelegate reservationFacade = SpringFacadeFactory.getReservationFacadeDelegate(); 

				reservationFacade.oweAddition(new OweTo(null,currentDate,price,vat,amount,discount,status,particular,room,invoiceNumber,invoiceSerial,concept));
			}

		}				

	}

}
