package gz.cyberbrain.cyberhotel.view.swing.edit;

import gz.cyberbrain.cyberhotel.controller.SaveAdapter;
import gz.cyberbrain.cyberhotel.controller.util.ErrorsHandler;
import gz.cyberbrain.cyberhotel.view.swing.FillableWindow;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.InfoPanel;
import gz.cyberbrain.cyberhotel.view.swing.util.locale.Locale;

import javax.swing.JInternalFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.UIManager;

import java.awt.Color;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.io.Serializable;
import java.util.HashMap;

import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.border.LineBorder;

public class BoardWindow extends FillableWindow {

	private static final long serialVersionUID = 1L;

	private static BoardWindow boardWindow = null;

	private SaveAdapter saveAdapter = null;

	private Serializable id = null;

	private JInternalFrame owner = null;

	private String dialogMessage = null;

	private String dialogTitle = null;

	private JPanel jContentPane = null;

	private JLabel codeLabel = null;

	private JTextField codeTextField = null;

	private JLabel descLabel = null;

	private JTextField descTextField = null;

	private JButton saveButton = null;

	private JButton cancelButton = null;

	private InfoPanel infoPanel = null;

	/**
	 * @param owner
	 */
	private BoardWindow() {
		super("",false,true,false,true);
		initialize();
	}

	public static BoardWindow getInstance() {
		if (boardWindow == null) boardWindow = new BoardWindow();
		return boardWindow;
	}

	public void setAdapter(SaveAdapter saveAdapter) {
		this.saveAdapter = saveAdapter;
	}

	public SaveAdapter getAdapter() {
		return saveAdapter;
	}

	public String getDialogMessage() {
		return dialogMessage;
	}

	public void setDialogMessage(String dialogMessage) {
		this.dialogMessage = dialogMessage;
	}

	public String getDialogTitle() {
		return dialogTitle;
	}

	public void setDialogTitle(String dialogTitle) {
		this.dialogTitle = dialogTitle;
	}


	public void setId(Serializable id) {
		this.id = id;
	}

	public Serializable getId() {
		return id;
	}

	public void setOwner(JInternalFrame owner) {
		this.owner = owner;
	}

	public JInternalFrame getOwner() {
		return owner;
	}


	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(629, 184);
		this.setLocation(387,0);
		this.setContentPane(getJContentPane());
		this.setResizable(false);		
		this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		this.addInternalFrameListener(new javax.swing.event.InternalFrameAdapter()
		{
			public void internalFrameClosing(javax.swing.event.InternalFrameEvent evt)
			{
				(BoardWindow.getInstance()).clean();
				(BoardWindow.getInstance()).dispose();

			}
		});
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			descLabel = new JLabel();
			descLabel.setBounds(new Rectangle(135, 70, 58, 16));
			descLabel.setText(Locale.get("BoardWindow.descLabel"));
			codeLabel = new JLabel();
			codeLabel.setBounds(new Rectangle(14, 70, 52, 16));
			codeLabel.setText(Locale.get("BoardWindow.codeLabel"));
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.add(codeLabel, null);
			jContentPane.add(getCodeTextField(), null);
			jContentPane.add(descLabel, null);
			jContentPane.add(getDescTextField(), null);
			jContentPane.add(getSaveButton(), null);
			jContentPane.add(getCancelButton(), null);
			jContentPane.add(getInfoPanel(),null);
		}
		return jContentPane;
	}

	/**
	 * This method initializes codeTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getCodeTextField() {
		if (codeTextField == null) {
			codeTextField = new JTextField();
			codeTextField.setBounds(new Rectangle(73, 70, 57, 19));
		}
		return codeTextField;
	}

	/**
	 * This method initializes descTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getDescTextField() {
		if (descTextField == null) {
			descTextField = new JTextField();
			descTextField.setBounds(new Rectangle(198, 70, 415, 19));
		}
		return descTextField;
	}

	/**
	 * This method initializes saveButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getSaveButton() {
		if (saveButton == null) {
			saveButton = new JButton();
			saveButton.setBounds(new Rectangle(413, 107, 93, 19));
			saveButton.setText(Locale.get("BoardWindow.saveButton"));
			saveButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					HashMap<String, Object> attrs = new HashMap<String, Object>();
					attrs.put("code",getCodeTextField().getText());
					attrs.put("description", getDescTextField().getText());					

					String message = getDialogMessage();
					String title = getDialogTitle();

					getAdapter().save(attrs);

					if (!inputErrors()) {

						Object[] options = {Locale.get("JOptionPane.Ok")};
						int n = JOptionPane.showOptionDialog(BoardWindow.getInstance(),
								message,
								title,
								JOptionPane.OK_OPTION,
								JOptionPane.INFORMATION_MESSAGE,
								null,
								options,
								options[0]);

						if (n == JOptionPane.OK_OPTION) {
							ListWindow listWindow = (ListWindow) getOwner();
							listWindow.loadList();
							clean();
							dispose();	                    	
						}

					}

				}

			});


		}
		return saveButton;
	}

	/**
	 * This method initializes cancelButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setBounds(new Rectangle(520, 107, 93, 19));
			cancelButton.setText(Locale.get("BoardWindow.cancelButton"));
			cancelButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					(BoardWindow.getInstance()).clean();
					(BoardWindow.getInstance()).dispose();

				}
			});
		}
		return cancelButton;
	}

	private InfoPanel getInfoPanel() {
		if (infoPanel == null) {
			infoPanel = new InfoPanel();
			infoPanel.setBounds(0,0,this.getWidth(),50);
		}
		return infoPanel;
	}

	public void setInfoPanel(InfoPanel ipanel) {		  

		jContentPane.remove(infoPanel);
		infoPanel = ipanel;
		jContentPane.add(infoPanel);

	}

	public void clean() {
		getInfoPanel().setInfoText(getInfoPanel().getInfoText());
		getCodeTextField().setEditable(true);
		getCodeTextField().setText("");
		getDescTextField().setText("");
		getCodeTextField().setBorder(UIManager.getBorder("TextField.border"));
		getDescTextField().setBorder(UIManager.getBorder("TextField.border"));
	}

	public void fill(HashMap fields) {

		getCodeTextField().setEditable(false);
		getCodeTextField().setText((String) fields.get("code"));
		getDescTextField().setText((String) fields.get("description"));

	}

	private boolean inputErrors() {

		if (!ErrorsHandler.getErrors().isEmpty()) {

			if (ErrorsHandler.getErrorAt(0).getItem().equals("code")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("BoardWindow.errors.code")+ "</html>");			
				getCodeTextField().setBorder(new LineBorder(Color.red, 2));
				Toolkit.getDefaultToolkit().beep();
				return true;
			}

			if (ErrorsHandler.getErrorAt(0).getItem().equals("description")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("BoardWindow.errors.description")+ "</html>");			
				getDescTextField().setBorder(new LineBorder(Color.red, 2));
				Toolkit.getDefaultToolkit().beep();
				return true;
			}


		}

		return false;

	}

}  //  @jve:decl-index=0:visual-constraint="10,10"
