package gz.cyberbrain.cyberhotel.view.swing.reception;

import java.util.Calendar;
import java.util.HashMap;
import java.util.Vector;

import gz.cyberbrain.cyberhotel.controller.RoomListAdapter;
import gz.cyberbrain.cyberhotel.controller.util.ErrorsHandler;
import gz.cyberbrain.cyberhotel.model.util.dates.DateOperations;
import gz.cyberbrain.cyberhotel.model.util.dates.Interval;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InvalidIntervalException;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.InfoPanel;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.PathHandler;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.RoomPanel;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.TextItem;
import gz.cyberbrain.cyberhotel.view.swing.util.locale.Locale;

import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.JInternalFrame;
import javax.swing.JScrollBar;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;

import javax.swing.JLabel;
import javax.swing.border.LineBorder;

import org.jdesktop.swingx.border.DropShadowBorder;

public class RoomPlanningWindow extends JInternalFrame {

	private static final long serialVersionUID = 1L;	
	private static RoomPlanningWindow roomPlanningWindow = null;
	private InfoPanel infoPanel = null;
	private JPanel jContentPane = null;
	private JPanel roomsScrollPane = null;
	private JPanel roomsPanel = null;	
	private JScrollBar roomsBar = null;
	private Vector rooms = null;
	private int numCols = 10;
	private int visRows = 4;
	private JPanel keysPanel = null;
	private JLabel keysLabel = null;
	private JLabel nonSmokersLabel = null;
	private JLabel smokersLabel = null;
	private JLabel dirtyLabel = null;
	private JLabel bioLabel = null;
	private String imgPath = PathHandler.getImagesPath();  //  @jve:decl-index=0:
	private JPanel statusPanel = null;
	private JLabel statusLabel = null;
	private JLabel freeLabel = null;
	private JLabel occupatedLabel = null;
	private JLabel lockedLabel = null;
	private JLabel whiteLabel = null;
	private JLabel orangeLabel = null;
	private JLabel grayLabel = null;

	/**
	 * This is the xxx default constructor
	 */
	private RoomPlanningWindow() {
		super("", false, true, false, true);
		initialize();
	}

	public static RoomPlanningWindow getInstance() {
		if(roomPlanningWindow == null) roomPlanningWindow = new RoomPlanningWindow();
		return roomPlanningWindow;
	}

	public InfoPanel getInfoPanel() {
		if (infoPanel == null) {
			infoPanel = new InfoPanel();
			infoPanel.setBounds(new Rectangle(0, 0, this.getWidth(), 50));
		}
		return infoPanel;
	}

	public void setInfoPanel(InfoPanel ipanel) {		  

		jContentPane.remove(infoPanel);
		infoPanel = ipanel;
		jContentPane.add(infoPanel);

	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(920, 683);		
		this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		this.setContentPane(getJContentPane());			
		this.addInternalFrameListener(new javax.swing.event.InternalFrameAdapter()
		{   

			public void internalFrameClosing(javax.swing.event.InternalFrameEvent evt)
			{        
				clean();
				roomPlanningWindow = null;
				
			}
		});	
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.add(getInfoPanel(), null);
			jContentPane.add(getRoomsScrollPane(), null);			
			jContentPane.add(getKeysPanel(), null);
			jContentPane.add(getStatusPanel(), null);
		}
		return jContentPane;
	}

	private Vector getRooms() {

		if (rooms == null) {
			RoomListAdapter adapter = new RoomListAdapter();
			rooms = adapter.load();
		}

		return rooms;

	}

	private int numOfRooms() {

		return getRooms().size();

	}

	private JPanel getRoomsPanel() {

		if (roomsPanel == null) {
			roomsPanel =  new JPanel();
			roomsPanel.setLayout(null);
			roomsPanel.setBounds(0,0,89*2,120*(numOfRooms()/numCols));	
		}

		return roomsPanel;

	}

	private void repaintRoomsPanel(int indexA) {

		RoomListAdapter adapter = new RoomListAdapter();
		TextItem item = null;
		HashMap map = null;
		int x = 0;
		int y = 0;
		int end = (numCols*visRows) + (numCols*indexA);

		getRoomsPanel().removeAll();

		if (end > getRooms().size()) end = getRooms().size();

		for (int i=numCols*indexA; i<end; i++) {
			RoomPanel panel = new RoomPanel();
			item = (TextItem) getRooms().elementAt(i);
			map = adapter.find(item.getId());
			panel.setRoomCode((String) map.get("code"));
			panel.setNumber((String) map.get("number"));
			panel.setSmokers(((Boolean) map.get("smokers")).booleanValue());
			panel.setClean(isClean((String) map.get("cleaningStatus")));
			panel.setInfected(isInfected((Calendar) map.get("checkingDate")));
			panel.setOccupancyStatus((String) map.get("occupancyStatus"));
			panel.setLocation(x*panel.getWidth(),y*panel.getHeight());
			panel.showStatus();
			getRoomsPanel().add(panel, null);		
			x++;
			if (x%numCols == 0) {
				y++;
				x=0;
			}
		}

		getRoomsPanel().repaint();

		getRoomsPanel().setVisible(false);
		getRoomsPanel().setVisible(true);

	}

	private JScrollBar getRoomsBar() {

		if (roomsBar == null) {

			int maxValue = (int) Math.ceil((double) numOfRooms() / (double) numCols);

			roomsBar = new JScrollBar(JScrollBar.VERTICAL, 0, visRows, 0, maxValue);
			roomsBar.setLocation(this.getWidth() - 20, 0);
			roomsBar.addAdjustmentListener(new AdjustmentListener() {

				public void adjustmentValueChanged(AdjustmentEvent e) {

					repaintRoomsPanel(e.getValue());

				}

			});	

		}

		return roomsBar;

	}

	private void addRoomsBar() {

		int maxValue = (int) Math.ceil((double) numOfRooms() / (double) numCols);

		if (maxValue > visRows) roomsScrollPane.add(getRoomsBar(), BorderLayout.EAST);

	}

	private boolean isClean(String cleaningStatus) {
		if (cleaningStatus.equals("Limpa")) return true;
		else return false;
	}

	private boolean isInfected(Calendar checkingDate) {
		boolean infected = false;
		Calendar today = Calendar.getInstance();
		try {
			if (DateOperations.getIntervalDifference(new Interval(checkingDate, today)) > 7) infected = true;
			else infected = false;
		} catch (InvalidIntervalException e) {			
			e.printStackTrace();
		}
		return infected;
	}

	/**
	 * This method initializes roomsScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JPanel getRoomsScrollPane() {
		if (roomsScrollPane == null) {
			roomsScrollPane = new JPanel();
			roomsScrollPane.setLayout(new BorderLayout());
			roomsScrollPane.setBounds(new Rectangle(0, 51, this.getWidth()-10, 120*visRows));

			roomsScrollPane.add(getRoomsPanel(), null);
			addRoomsBar();

			repaintRoomsPanel(0);

		}
		return roomsScrollPane;
	}

	private void clean() {
		jContentPane = null;
		roomsScrollPane = null;
	}

	/**
	 * This method initializes keysPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getKeysPanel() {
		if (keysPanel == null) {
			bioLabel = new JLabel(Locale.get("RoomPlanningWindow.bioLabel"), getHealthIcon(), JLabel.LEFT);
			bioLabel.setBounds(new Rectangle(93, 66, 85, 34));			
			dirtyLabel = new JLabel(Locale.get("RoomPlanningWindow.dirtyLabel"), getTrashIcon(), JLabel.LEFT);
			dirtyLabel.setBounds(new Rectangle(8, 66, 85, 34));
			smokersLabel = new JLabel(Locale.get("RoomPlanningWindow.smokersLabel"), getSmokingIcon(), JLabel.LEFT);
			smokersLabel.setBounds(new Rectangle(93, 27, 85, 34));
			nonSmokersLabel = new JLabel(Locale.get("RoomPlanningWindow.nonSmokersLabel"), getNonSmokingIcon(), JLabel.LEFT);
			nonSmokersLabel.setBounds(new Rectangle(8, 27, 85, 34));
			keysLabel = new JLabel();
			keysLabel.setText("<html> <b>" + Locale.get("RoomPlanningWindow.keysLabel") + "</html>");
			keysLabel.setBounds(new Rectangle(7, 7, 54, 16));
			keysPanel = new JPanel();
			keysPanel.setLayout(null);
			keysPanel.setBounds(new Rectangle(7, 538, 186, 110));
			DropShadowBorder dsb = new DropShadowBorder();
			keysPanel.setBorder(new DropShadowBorder(dsb.getShadowColor(), dsb.getShadowSize(),
					dsb.getShadowOpacity(), dsb.getCornerSize(), true, true, true, true));
			keysPanel.add(keysLabel, null);
			keysPanel.add(nonSmokersLabel, null);
			keysPanel.add(smokersLabel, null);
			keysPanel.add(dirtyLabel, null);
			keysPanel.add(bioLabel, null);
		}
		return keysPanel;
	}

	private ImageIcon getNonSmokingIcon() {
		ImageIcon nonSmoking = new ImageIcon(imgPath + "nosmoking.png");
		return new ImageIcon(nonSmoking.getImage().
				getScaledInstance(25, 25, Image.SCALE_SMOOTH));
	}

	private ImageIcon getSmokingIcon() {
		ImageIcon smoking  = new ImageIcon(imgPath + "smoking.png");
		return new ImageIcon(smoking.getImage().
				getScaledInstance(25, 25, Image.SCALE_SMOOTH));
	}

	private ImageIcon getTrashIcon() {
		ImageIcon trash = new ImageIcon(imgPath + "vacuum.png"); 
		return new ImageIcon(trash.getImage().
				getScaledInstance(25, 25, Image.SCALE_SMOOTH));
	}

	private ImageIcon getHealthIcon() {
		ImageIcon health = new ImageIcon(imgPath + "biohazard.png"); 
		return new ImageIcon(health.getImage().
				getScaledInstance(25, 25, Image.SCALE_SMOOTH));
	}

	/**
	 * This method initializes statusPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getStatusPanel() {
		if (statusPanel == null) {
			grayLabel = new JLabel();
			grayLabel.setBounds(new Rectangle(11, 78, 52, 22));
			grayLabel.setOpaque(true);
			grayLabel.setBackground(Color.BLUE);			
			grayLabel.setBorder(new LineBorder(Color.BLACK));
			orangeLabel = new JLabel();
			orangeLabel.setBounds(new Rectangle(11, 53, 52, 22));
			orangeLabel.setOpaque(true);
			orangeLabel.setBackground(Color.RED);			
			orangeLabel.setBorder(new LineBorder(Color.BLACK));
			whiteLabel = new JLabel();
			whiteLabel.setBounds(new Rectangle(11, 29, 52, 22));
			whiteLabel.setOpaque(true);
			whiteLabel.setBackground(Color.GREEN);
			whiteLabel.setBorder(new LineBorder(Color.BLACK));
			lockedLabel = new JLabel();
			lockedLabel.setBounds(new Rectangle(67, 82, 56, 16));
			lockedLabel.setText(Locale.get("RoomPlanningWindow.lockedLabel"));
			occupatedLabel = new JLabel();
			occupatedLabel.setBounds(new Rectangle(67, 58, 56, 16));
			occupatedLabel.setText(Locale.get("RoomPlanningWindow.occupatedLabel"));
			freeLabel = new JLabel();
			freeLabel.setBounds(new Rectangle(67, 33, 56, 16));
			freeLabel.setText(Locale.get("RoomPlanningWindow.freeLabel"));
			statusLabel = new JLabel();
			statusLabel.setText("<html> <b>" + Locale.get("RoomPlanningWindow.statusLabel") + "</html>");
			statusLabel.setBounds(new Rectangle(7, 7, 52, 16));			
			statusPanel = new JPanel();
			statusPanel.setLayout(null);
			statusPanel.setBounds(new Rectangle(194, 538, 131, 110));
			DropShadowBorder dsb = new DropShadowBorder();
			statusPanel.setBorder(new DropShadowBorder(dsb.getShadowColor(), dsb.getShadowSize(),
					dsb.getShadowOpacity(), dsb.getCornerSize(), true, true, true, true));
			statusPanel.add(statusLabel, null);
			statusPanel.add(freeLabel, null);
			statusPanel.add(occupatedLabel, null);
			statusPanel.add(lockedLabel, null);
			statusPanel.add(whiteLabel, null);
			statusPanel.add(orangeLabel, null);
			statusPanel.add(grayLabel, null);
		}
		return statusPanel;
	}

	public void showErrors() {
		
		if (!ErrorsHandler.getErrors().isEmpty()) {
			
			if (ErrorsHandler.getErrorAt(0).getItem().equals("roomBusy")) {
				
				(RoomPlanningWindow.getInstance()).getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("RoomPlanningWindow.errors.roomBusy")+ "</html>");

				Toolkit.getDefaultToolkit().beep();

			}
		}
	}

	public void cleanErrors() {

		getInfoPanel().setInfoText(getInfoPanel().getInfoText());

	}	

}
